/*
io_sd_card.c

Hardware routines for interfacing with SD cards on a Ninjapass X9

Copyright (C) 2007 by CJ Bell
siegebell@gmail.com


This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
Boston, MA  02110-1301, USA.
*/

#ifndef INCLUDED_io_sd_card_h
#define INCLUDED_io_sd_card_h

#ifdef __cplusplus
extern "C" {
#endif


/* SD commands */
typedef enum
    { SetIdle                =  0 // CMD0
    , GetCID                 =  2 // CMD2   
    , GetRCA                 =  3 //         Get 
    , SendOPCommand          =  5 // IO_SEND_OP_COND (CMD5)
    , SelectCard             =  7 //
    , GetCSD                 =  9 // CMD9    Get Card Specific Data
    , AbortReadWrite         = 12 // CMD12   Data Transfer Abort
    , GetStatus              = 13 //
    , GoInactiveState        = 15 //
    , SetBlockLength         = 16 // CMD16
    , ReadSingleBlock        = 17 //
    , ReadMultipleBlocks     = 18 //
    , WriteSingleBlock       = 24 //
    , WriteMultipleBlocks    = 25 //
    , AppCommand             = 55 // APPCMD
    } SDCommand;

typedef enum
    { SetBusWidth  = 6  //
    , ACMD41       = 41 // ACMD41
    } SDAppCommand;

typedef enum
    { SDIdleState   = 0
    , SDReadyState
    , SDIDState
    , SDStandbyState
    , SDTransferState
    , SDDataState
    , SDReceiveState
    , SDProgrammingState
    , SDDisconnectedState
    , SDInactiveState
    } SDState;

typedef struct
{
    const uint8  csdStructure              :  2;
    const uint8  reserved1                 :  6;
    const uint8  dataReadAccessTime1       :  8;  // TAAC
    const uint8  dataReadAccessTime2       :  8;  // NSAC
    const uint8  maxDataTransferRate       :  8;
    const uint16 cardCommandClasses        :  12;
    const uint8  maxReadDataBlockLength    :  4;
    const uint8  allowPartialRead          :  1;
    const uint8  allowWriteMisalign        :  1;
    const uint8  allowReadMisalign         :  1;
    const uint8  dsrImplemented            :  1;
    const uint8  reserved2                 :  2;
    const uint16 deviceSize                : 12;
    const uint8  maxReadCurrentMin         :  3;
    const uint8  maxReadCurrentMax         :  3;
    const uint8  maxWriteCurrentMin        :  3;
    const uint8  maxWriteCurrentMax        :  3;
    const uint8  sizeMultiplier            :  3;
    const uint8  allowEraseSingleBlock     :  1;
    const uint8  eraseSectorSize           :  7;
    const uint8  writeProtectGroupSize     :  7;
    const uint8  writeProtectGroupEnabled  :  1;
    const uint8  reserved3                 :  2;
    const uint8  writeSpeedFactor          :  3;
    const uint8  maxWriteLength            :  4;
    const uint8  allowPartialWrite         :  1;
    const uint8  reserved4                 :  5;
    const uint8  fileFormatGroup           :  1;
    const uint8  copyFlag                  :  1;
    const uint8  permanentWriteProtection  :  1;
          uint8  temporaryWriteProtection  :  1;
    const uint8  fileFormat                :  2;
          uint8  reserved5                 :  2;
          uint8  crc7                      :  7;  // Used to ensure this header was transferred properly
    const uint8  direction                 :  1;  // 1 == from card to host
} CSDRegister;

typedef struct
{
    bool outOfRange                : 1;
    bool addressError              : 1;
    bool blockLengthError          : 1;
    bool eraseSequenceError        : 1;
    bool eraseParameter            : 1;
    bool writeProtectViolation     : 1;
    bool cardIsLocked              : 1;
    bool unlockFailed              : 1;
    bool crcCheckFailure           : 1;
    bool illegalCommand            : 1;
    bool cardECCFailed             : 1;
    bool ccErrornge                : 1;
    bool error                     : 1;
    bool reserved1                 : 1;
    bool reserved2                 : 1;
    bool invalidOverwrite          : 1;
    bool writeProtectEraseSkip     : 1;
    bool cardECCDisabled           : 1;
    bool eraseReset                : 1;
    bool currentState              : 4;
    bool readyForData              : 1;
    bool reserved3                 : 2;
    bool appCommand                : 1;
    bool reserved4                 : 1;
    bool authSequenceError         : 1;
    bool reserved5                 : 1;
    bool reserved6                 : 2;
} SDCardStatus;

typedef struct
{
    bool   start                     :  1;
    bool   direction                 :  1;
    bool   commandIndex              :  6;
    uint16 newRCA                    : 16;
    uint16 cardStatusError           : 16;
    bool   crc7                      :  7;
    bool   end                       :  1;
} SDResponseR6;



void SDSendCommand(SDCommand sdCmd, uint32 arg, uint8* response);
bool SDInitialize(unsigned int timeout, uint32* relativeCardAddress);
bool SDWriteSingleBlock(uint32 address, const void* buffer);
bool SDReadSingleBlock(uint32 address, void* destination);

static inline void SDSendAppCommand(SDAppCommand sdAppCmd, uint32 arg, uint8* response)
{
    SDSendCommand((SDCommand)sdAppCmd,arg,response);
}

#ifdef __cplusplus
}
#endif

#endif // INCLUDED_io_sd_card_h
